package com.ibm.sysmgt.raidmgr.cim.provider.association;

import com.ibm.sysmgt.raidmgr.cim.provider.*;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider20.AssociatorProvider;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.client.ProviderCIMOMHandle;

public abstract class RAIDAssociatorProviderBase extends RAIDProvider implements AssociatorProvider, InstanceProvider
{
   protected static PrintWriter associationLog = setupLog("RAIDAssociationProvider.log");

   public String CIM_CREATION_CLASS_NAME = "NULL";
   public static final String ANTECEDENT = "Antecedent";
   public static final String DEPENDENT  = "Dependent";

   public RAIDAssociatorProviderBase(String logHeader, String creationClassName)
   {
      super(logHeader);
      CIM_CREATION_CLASS_NAME = creationClassName;
   }

   public Vector associators(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                             String role, String resultRole, boolean includeQaulifiers, boolean includeClassOrigin,
                             String[] propertyList) throws CIMException
   {
      associationLog.println(logHeader + "associators()");

      Vector associators = new Vector();
      Vector instances = associatorHelper(assocName, objectName, resultClass, role, resultRole);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         associators.addElement(ci.filterProperties(propertyList, includeQaulifiers, includeClassOrigin));
      }

      return associators;
   }

   public Vector associatorNames(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                                 String role, String resultRole) throws CIMException
   {
      associationLog.println(logHeader + "associatorNames()");

      Vector names = new Vector();
      Vector instances = associatorHelper(assocName, objectName, resultClass, role, resultRole);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         // Create a vector of Names from the instances returned from the helper method
         CIMInstance instance = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(instance.getClassName(), objectName.getNameSpace());
         cop.setKeys(instance.getKeyValuePairs());
         names.addElement(cop);
      }

      return names;
   }
   
   protected abstract Vector associatorHelper(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                                 String role, String resultRole) throws CIMException;

   public Vector references(CIMObjectPath assocName, CIMObjectPath objectName,
                            String role, boolean includeQaulifiers, boolean includeClassOrigin,
                            String[] propertyList) throws CIMException
   {
      associationLog.println(logHeader + "references()");

      Vector references = new Vector();
      Vector instances = referenceHelper(assocName, objectName, role);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         references.addElement(ci.filterProperties(propertyList, includeQaulifiers, includeClassOrigin));
      }

      return references;
   }

   public Vector referenceNames(CIMObjectPath assocName, CIMObjectPath objectName, String role) throws CIMException
   {
      associationLog.println(logHeader + "referenceNames()");

      Vector names = new Vector();
      Vector instances = referenceHelper(assocName, objectName, role);

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         // Create a vector of Names from the instances returned from the helper method
         CIMInstance instance = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(instance.getClassName(), objectName.getNameSpace());
         cop.setKeys(instance.getKeyValuePairs());
         names.addElement(cop);
      }

      return names;
   }

   protected abstract Vector referenceHelper(CIMObjectPath assocName, CIMObjectPath objectName, String role) throws CIMException;

   //*************************************************************************************************************
   // Instance Provider Implementation
   //    Windows does not support the AssociatorProvider interface but instead wants to enumerate instances of
   //    the association classes.
   //*************************************************************************************************************
   
   /** Satisfies the InstanceProvider interface */
   public CIMObjectPath createInstance(CIMObjectPath op, CIMInstance ci) throws CIMException
   {
      associationLog.println(logHeader + "createInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }
   
   /** Satisfies the InstanceProvider interface */
   public void deleteInstance(CIMObjectPath op) throws CIMException
   {
      associationLog.println(logHeader + "deleteInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }
   
   /** Satisfies the InstanceProvider interface
     * Enumerates only the names of instances of the class specified by op
     */
   public abstract Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException;
   
   /** Satisfies the InstanceProvider interface
     * Enumerates the instances of the class specified by op
     */
   public abstract Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException;
   
   /** Satisfies the InstanceProvider interface */
   public Vector execQuery(CIMObjectPath op, String query, int ql, CIMClass cc) throws CIMException
   {
      associationLog.println(logHeader + "execQuery()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }
   
   /** Satisfies the InstanceProvider interface */
   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc, boolean localOnly) throws CIMException
   {
      associationLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");

      if (!op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         associationLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      CIMObjectPath antecedent = (CIMObjectPath)getKeyValue(op, "Antecedent");
      CIMObjectPath dependent = (CIMObjectPath)getKeyValue(op, "Dependent");

      CIMInstance instance = cc.newInstance();

      instance.setProperty("Antecedent", new CIMValue((CIMObjectPath) getKeyValue(op, "Antecedent")));
      instance.setProperty("Dependent", new CIMValue((CIMObjectPath) getKeyValue(op, "Dependent")));

      return instance;
   }
   
   /** Satisfies the InstanceProvider interface */
   public void setInstance(CIMObjectPath op, CIMInstance ci) throws CIMException
   {
      associationLog.println(logHeader + "setInstance()");
      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

}
